#!/usr/bin/env python

"An iterator merging class similar to heapq.merge in Python 2.6."

from bisect import insort_right

class itermerge:

    """
    Merge ordered sequences to produce an ordered, combined sequence of
    results.
    """

    def __init__(self, sequences):
        self.iters = []

        # Prepare the underlying iterators.

        for seq in sequences:
            it = iter(seq)
            next = it.next
            self._add_next(next)

    def sort(self):
        pass # The output should be sorted.

    def __iter__(self):
        return self

    def _add_next(self, next):

        """
        Store the current value for an iterator, alongside the means of
        getting the next value: the 'next' method.
        """

        try:
            insort_right(self.iters, (next(), next))
        except StopIteration:
            pass

    def next(self):
        if self.iters:
            value, next = self.iters[0]
            del self.iters[0]
            self._add_next(next)
            return value
        else:
            raise StopIteration

# vim: tabstop=4 expandtab shiftwidth=4
